{% if offline %}
    {% if inline %}
        <script>
            {% include 'wrapper/assets/jquery-1.12.4.min.js' %}
            {% include 'wrapper/assets/bootstrap.min.js' %}
        </script>
        <script>
        {% include 'wrapper/assets/script.js' %}
        </script>
    {% else %}
        <script src="{{ file_name }}_assets/js/jquery-1.12.4.min.js"></script>
        <script src="{{ file_name }}_assets/js/bootstrap.min.js"></script>
        <script src="{{ file_name }}_assets/js/script.js"></script>
    {% endif %}
{% else %}
    <script src="https://code.jquery.com/jquery-1.12.4.min.js"></script>
    <script src="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/js/bootstrap.min.js" integrity="sha384-Tc5IQib027qvyjSMfHjOMaLkfuWVxZxUPnCJA7l2mCWNIpG9mGCD8wGNIcPD7Txa" crossorigin="anonymous"></script>
    <script>
    {% include 'wrapper/assets/script.js' %}
    </script>
{% endif %}