{% if name %}
    <p class="h4">{{ name }}</p>
{% endif %}
<table class="table table-condensed stats">
    <tbody>
        {% for row in rows %}
        <tr{% if 'alert' in row and row['alert'] %} class="alert"{% endif %}>
            <th>{{ row['name'] }}</th>
            <td>{{ row['value'] }}</td>
        </tr>
        {% endfor %}
    </tbody>
</table>
{% if caption %}
    <div class="caption text-center text-muted">
        {{ caption }}
    </div>
{% endif %}