{% if offline %}
    {%- if inline -%}
        <style>
        {% if theme is not none %}
            {% if theme.value == 'flatly' %}
                {% include 'wrapper/assets/flatly.bootstrap.min.css' %}
            {% elif theme.value == 'united' %}
                {% include 'wrapper/assets/united.bootstrap.min.css' %}
            {% endif %}
        {% else %}
            {% include 'wrapper/assets/bootstrap.min.css' %}
            {% include 'wrapper/assets/bootstrap-theme.min.css' %}
        {% endif %}
        </style>
    {% else %}
        {% if theme is not none %}
            {% if theme.value == 'flatly' %}
                <link href="{{ file_name }}_assets/css/flatly.bootstrap.min.css" rel="stylesheet" />
            {% elif theme.value == 'united' %}
                <link href="{{ file_name }}_assets/css/united.bootstrap.min.css" rel="stylesheet" />
            {% endif %}
        {% else %}
            <link rel="stylesheet" href="{{ file_name }}_assets/css/bootstrap.min.css" />
            <link rel="stylesheet" href="{{ file_name }}_assets/css/bootstrap-theme.min.css" />
        {% endif %}
    {% endif %}
{% else %}
    {% if theme is not none %}
        {% if theme.value == 'flatly' %}
            <link href="https://maxcdn.bootstrapcdn.com/bootswatch/3.3.7/flatly/bootstrap.min.css" rel="stylesheet" integrity="sha384-+ENW/yibaokMnme+vBLnHMphUYxHs34h9lpdbSLuAwGkOKFRl4C34WkjazBtb7eT" crossorigin="anonymous" />
        {% elif theme.value == 'united' %}
            <link href="https://maxcdn.bootstrapcdn.com/bootswatch/3.3.7/united/bootstrap.min.css" rel="stylesheet" crossorigin="anonymous" />
        {% endif %}
    {% else %}
        <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/css/bootstrap.min.css" integrity="sha384-BVYiiSIFeK1dGmJRAkycuHAHRg32OmUcww7on3RYdg4Va+PmSTsz/K68vbdEjh4u" crossorigin="anonymous" />
        <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/css/bootstrap-theme.min.css" integrity="sha384-rHyoN1iRsVXV4nD0JutlnGaslCJuC7uwjduW9SVrLvRYooPp2bWYgmgJQIXwl/Sp" crossorigin="anonymous" />
    {% endif %}
{% endif %}

{%- if inline -%}
    <style>{% include 'wrapper/assets/style.css' %}</style>
{% else %}
    <link rel="stylesheet" href="{{ file_name }}_assets/css/style.css" />
{% endif %}