from pandas_profiling.report.presentation.flavours.html.alerts import HTMLAlerts
from pandas_profiling.report.presentation.flavours.html.collapse import HTMLCollapse
from pandas_profiling.report.presentation.flavours.html.container import HTMLContainer
from pandas_profiling.report.presentation.flavours.html.duplicate import HTMLDuplicate
from pandas_profiling.report.presentation.flavours.html.frequency_table import (
    HTMLFrequencyTable,
)
from pandas_profiling.report.presentation.flavours.html.frequency_table_small import (
    HTMLFrequencyTableSmall,
)
from pandas_profiling.report.presentation.flavours.html.html import HTMLHTML
from pandas_profiling.report.presentation.flavours.html.image import HTMLImage
from pandas_profiling.report.presentation.flavours.html.root import HTMLRoot
from pandas_profiling.report.presentation.flavours.html.sample import HTMLSample
from pandas_profiling.report.presentation.flavours.html.table import HTMLTable
from pandas_profiling.report.presentation.flavours.html.toggle_button import (
    HTMLToggleButton,
)
from pandas_profiling.report.presentation.flavours.html.variable import HTMLVariable
from pandas_profiling.report.presentation.flavours.html.variable_info import (
    HTMLVariableInfo,
)

__all__ = [
    "HTMLCollapse",
    "HTMLContainer",
    "HTMLDuplicate",
    "HTMLFrequencyTable",
    "HTMLFrequencyTableSmall",
    "HTMLHTML",
    "HTMLImage",
    "HTMLRoot",
    "HTMLSample",
    "HTMLTable",
    "HTMLToggleButton",
    "HTMLVariable",
    "HTMLVariableInfo",
    "HTMLAlerts",
]
