<a class="anchor-pos anchor-pos-variable" id="pp_var_{{ anchor_id }}"></a>
<div class="variable{% if ignore %} ignore{% endif %}">
    {{ top.render() }}

    {% if bottom is not none %}
        {{ bottom.render() }}
    {% endif %}
</div>