{% if nav == True %}
body {
  padding-top: 80px;
}
{% endif %}
.content .container-fluid{
    margin-left: 20px !important;
    margin-right: 20px !important;
    margin-bottom: 20px;
}

.page-header{
    border:0 !important;
}

.row.variable, .section-items > .row {
    border: 1px solid #e1e1e8;
    border-top: hidden;
}

.row.spacing{
    padding: 2em 1em;
}

.row.header {
    border-bottom: 1px solid #e1e1e8;
    /*background-color: #f5f5f5;*/
    padding-left: 2em;
}


.dl-horizontal dt {
    text-align: left;
    padding-right: 1em;
    white-space: normal;
}

.dl-horizontal dd {
    margin-left: 0;
}

.col-md-12 {
    padding-left: 2em;
}

.indent {
    margin-left: 1em;
}

.center-img {
    margin-left: auto !important;
    margin-right: auto !important;
    display: block;
}

/* Table example_values */
table.example_values {
    border: 0;
}

.example_values th {
    border: 0;
    padding: 0;
    color: #555;
    font-weight: 600;
}

.example_values tr, .example_values td {
    border: 0;
    padding: 0;
    color: #555;
}

/* STATS */
table.stats, table.sample, table.duplicate{
    border: 0;
}

.stats tr, .sample tr, .duplicate tr {
    border: 0;
}

.stats th, .stats td{
    color: #555;
    border: 0;
}

.stats th {
    padding: 0 2em 0 0;
    font-weight: 600;
}

.stats td {
    padding: 1px;
}


/* Sample table */
table.sample, table.duplicate{
    margin-bottom: 2em;
    margin-left: 1em;
}

.sample td, .sample th, .duplicate td, .duplicate th {
    padding: 0.5em;
    white-space: nowrap;
    border: 0;

}

.sample thead, .duplicate thead {
    border-top: 0;
    border-bottom: 2px solid #ddd;
}

.sample td, .duplicate td {
    width: 100%;
}


/* There is no good solution available to make the divs equal height and then center ... */
.histogram {
    margin-top: 3em;
}

/* Freq table */
table.freq {
    margin-bottom: 2em;
    border: 0;
}

table.freq th, table.freq tr, table.freq td {
    border: 0;
    padding: 0;
}

.freq thead {
    font-weight: 600;
    white-space: nowrap;
    overflow: hidden;
    text-overflow: ellipsis;

}

/* Freq mini */
.freq.mini td {
    width: 50%;
    padding: 1px;
    font-size: 12px;

}

table.freq.mini {
    width: 100%;
}

.freq.mini th {
    overflow: hidden;
    text-overflow: ellipsis;
    white-space: nowrap;
    max-width: 5em;
    font-weight: 400;
    text-align: right;
    padding-right: 0.5em;
}

/* Message classes */
.missing {
    color: #a94442;
}

.alert, .alert > th, .alert > td {
    color: #a94442;
}

.ignore {
    opacity: 0.4;
}

/* Bars in tables */
.freq.table{
    table-layout: fixed;
}

.freq:not(.mini) tr td:nth-child(1), .freq:not(.mini) tr th:nth-child(1){
    width: auto;
    max-width: none;

    white-space: nowrap;
    overflow: hidden;
    text-overflow: ellipsis;
}

.freq:not(.mini) tr td:nth-child(2), .freq:not(.mini) tr td:nth-child(3), .freq:not(.mini) tr th:nth-child(2), .freq:not(.mini) tr th:nth-child(3){
    width: 100px;
    text-align: right;
}
.freq:not(.mini) tr td:nth-child(4), .freq:not(.mini) tr th:nth-child(4){
    width:200px;
}

.freq .bar {
    float: left;
    width: 0;
    height: 100%;
    line-height: 20px;
    color: #fff;
    text-align: center;
    background-color: {{ primary_color }};
    border-radius: 3px;
    margin-right: 4px;
}

.other .bar {
    background-color: #999;
}

.missing .bar {
    background-color: #a94442;
}

.tooltip-inner {
    width: 100%;
    white-space: nowrap;
    text-align: left;
}

.extrapadding {
    padding: 2em;
}

.variable .h4 {
    text-overflow: ellipsis;
    display:inline-block;
    width: calc(90%); /* The trick is here! */
    overflow:hidden;
}

.variable.ignore .h4{
    text-decoration: line-through;
}

.table-responsive{
    overflow: scroll;
    width: 100%;
    overflow-y: hidden;
}
.img-responsive{
    max-width: 99%;
}
.footer-text{
    padding:20px;
}

table.list-warnings td{
    padding-right:10px;
}

a.anchor-pos {
    display: block;
    position: relative;
    top: -70px;
    visibility: hidden;
}

a.anchor-pos-variable{
    /*top: -70px;*/
}

#sample-container, #duplicate-container{
    overflow: auto;
    width: 100%;
    overflow-y: hidden;
}

#overview-content td, #overview-content th{
    border-top: 0;
    line-height: 1;
}

.variable-description{
    color: #777;
    font-size: 10pt;
    margin-top: 10px;
    font-style: italic;
}

select.multiple{
    width: 180px;
    height: 500px;
    margin: 10px 0;
}

.named-list-item{
    padding: 1em;
}

/* not printing tabs */
@media print {
    .tab-content > .tab-pane, .collapse {
        display: block !important;
        opacity: 1 !important;
        visibility: visible !important;
        /*page-break-after: always;*/
        page-break-after: right;
        page-break-before: avoid;
    }

    .nav-pills, .nav-tabs, button[data-toggle="collapse"], .mini, .col-sm-3 img {
        display:none !important;
    }

    a[download="config.yml"]:after {
        content: none !important;
    }

    .row {
        border: 0 !important;
    }
}

.text-placeholder {
    display: inline-block;
    background-color: #444;
    height: 12px;
    border-radius: 100px;
    margin: 5px 0;
    min-width: 200px;
    opacity: .1;
}